{ *************************************************************************** }
{                                                                             }
{ Hotzonu 2 DOLIB Manager Unit                                                }
{                                                                             }
{ 2001-04 Hotzonu Project, Monazilla Software Foundation 2001                 }
{                                                                             }
{ *************************************************************************** }

unit DolibModule;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  HTTPApp, Dolib;

type
  TDolibManager = class(TDataModule)
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
  private
    { Private 錾 }
    FDolib : TDolib;
    FUserAgent: string;
    FSessionID: string;
    FDisconnectedEvt: TNotifyEvent;
    FConnectedEvt: TNotifyEvent;
    function GetUserAgent: string;
    function GetSessionID: string;
    function GetConnected: boolean;
  public
    { Public 錾 }
    function  Connect: boolean;
    function  Disconnect: boolean;
    procedure ShowErrorMsg;
    property  UserAgent: string   read  GetUserAgent;
    property  SessionID: string   read  GetSessionID;
    property  Connected: boolean  read  GetConnected;
    property  OnConnected: TNotifyEvent read  FConnectedEvt write FConnectedEvt;
    property  OnDisconnected: TNotifyEvent read  FDisconnectedEvt write FDisconnectedEvt;
  end;

var
  DolibManager: TDolibManager;

implementation

{$R *.DFM}

uses
  Hzbconst, ZonuConf, Logindlg2ch;

{ TDataModule1 }

procedure TDolibManager.DataModuleCreate(Sender: TObject);
begin
  FUserAgent  :=  '';
  FSessionID  :=  '';
  FDolib              :=  TDolib.Create;
end;

procedure TDolibManager.DataModuleDestroy(Sender: TObject);
begin
  if  (FDolib.Connected)  then  begin
    Disconnect;
  end;
  FDolib.Free;
end;

function TDolibManager.GetUserAgent: string;
begin
  if  (FUserAgent = '') then  begin
    if  (FDolib.Connected)  then  begin
      FUserAgent  :=  FDolib.UserAgent + DOLIB_USER_AGENT;
    end else begin
      FUserAgent  :=  Format('Monazilla/%s (%s)', [FDolib.Version, DOLIB_USER_AGENT]);
    end;
  end;
  Result      :=  FUserAgent;
  {
  if  (FUserAgent = '') then  begin
    if  (not FDolib.Connected)  then  begin
      if  (not FDolib.Connect)  then  begin
        raise Exception.Create(FDolib.ErrorMsg);
      end;
    end;
    FUserAgent  :=  FDolib.UserAgent + DOLIB_USER_AGENT;
  end;
  Result      :=  FUserAgent;
  }
end;

function TDolibManager.GetSessionID: string;
begin
  if  (FSessionID = '') then  begin
    if  (not FDolib.Connected)  then  begin
      if  (not FDolib.Connect)  then  begin
        raise Exception.Create(FDolib.ErrorMsg);
      end;
    end;
    FSessionID  :=  HTTPApp.HTTPEncode(FDolib.SessionID);
  end;
  Result  :=  FSessionID;
end;


function TDolibManager.GetConnected: boolean;
begin
  Result  :=  FDolib.Connected;
end;


function TDolibManager.Connect: boolean;
begin
  Result  :=  False;
  if  (not FDolib.Connected)  then  begin
    LoginDialog2ch  :=  TLoginDialog2ch.Create(Self);
    if  (ZonuConfig.DolibShowDialog) or (ZonuConfig.DolibUserName = '')  then  begin
      if  (LoginDialog2ch.ShowModal <> mrOk) then  begin
        LoginDialog2ch.Close;
        Exit;
      end;
    end;
    LoginDialog2ch.Animate1.Active  :=  True;
    LoginDialog2ch.Edit1.Enabled    :=  False;
    LoginDialog2ch.Edit2.Enabled    :=  False;
    LoginDialog2ch.Button1.Enabled  :=  False;
    LoginDialog2ch.Button2.Enabled  :=  False;
    LoginDialog2ch.Cursor :=  crHourGlass;
    LoginDialog2ch.Show;
    LoginDialog2ch.Update;
    FDolib.ClientUA     :=  DOLIB_USER_AGENT;
    FDolib.UserName     :=  ZonuConfig.DolibUserName;
    FDolib.Password     :=  ZonuConfig.DolibPassword;
    if  (ZonuConfig.DolibUseProxy)  then  begin
      FDolib.ProxyAddress :=  ZonuConfig.DolibProxyAddress;
      FDolib.ProxyPort    :=  ZonuConfig.DolibProxyPort;
    end;
    if  (not FDolib.Connect)  then  begin
      MessageDlg(FDolib.ErrorMsg, mtError, [mbOk], 0);
      FDolib.Disconnect;
    end else begin
      Result  :=  True;
      if  (Assigned(FConnectedEvt)) then  begin
        FConnectedEvt(Self);
      end;
    end;
    LoginDialog2ch.Release;
  end;
end;

function TDolibManager.Disconnect: boolean;
begin
  Result      :=  FDolib.Disconnect;
  FUserAgent  :=  '';
  FSessionID  :=  '';
  if  (Assigned(FDisconnectedEvt)) then  begin
    FDisconnectedEvt(Self);
  end;
end;

procedure TDolibManager.ShowErrorMsg;
begin
  MessageDlg(FDolib.ErrorMsg, mtError, [mbOk], 0);
end;

end.
