{ *************************************************************************** }
{                                                                             }
{ Hotzonu 2 JPEG Popup Window Unit                                            }
{                                                                             }
{ 2001-04 Hotzonu Project, Monazilla Software Foundation 2001                 }
{                                                                             }
{ *************************************************************************** }

unit Pictpop;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ComCtrls, HttpProt, Jpeg, Buttons, Menus;

type
  TPicturePopup = class(TForm)
    HttpCli1: THttpCli;
    Panel1: TPanel;
    StatusBar1: TStatusBar;
    Panel2: TPanel;
    btnResize: TSpeedButton;
    Panel3: TPanel;
    TrackBar1: TTrackBar;
    Image1: TImage;
    PopupMenu1: TPopupMenu;
    SaveAs1: TMenuItem;
    SaveDialog1: TSaveDialog;
    N1: TMenuItem;
    N2: TMenuItem;
    procedure HttpCli1RequestDone(Sender: TObject; RqType: THttpRequest;
      Error: Word);
    procedure btnResizeClick(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SaveAs1Click(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure N2Click(Sender: TObject);
  private
    { Private 錾 }
    FHttpStream : TMemoryStream;
    jpgData : TJpegImage;
    procedure SaveSetting;
  public
    { Public 錾 }
    procedure Load(const FileName: string);
  end;

var
  PicturePopup: TPicturePopup;

implementation

{$R *.dfm}

uses
  ZonuConf, Hzbconst;

{ TPicturePopup }

procedure TPicturePopup.Load(const FileName: string);
begin
  jpgData                   :=  TJpegImage.Create;
  FHttpStream :=  TMemoryStream.Create;
  if  (ZonuConfig.ProxyUse) then  begin
    HttpCli1.Proxy          :=  ZonuConfig.Proxy;
    HttpCli1.ProxyPort      :=  ZonuConfig.ProxyPort;
    HttpCli1.ProxyUsername  :=  ZonuConfig.ProxyUserName;
    HttpCli1.ProxyPassword  :=  ZonuConfig.ProxyPassword;
  end;
  HttpCli1.Agent            :=  USER_AGENT;
  FHttpStream.Position      :=  0;
  FHttpStream.Size          :=  0;
  HttpCli1.RcvdStream       :=  FHttpStream;
  HttpCli1.URL              :=  FileName;
  HttpCli1.GetASync;
end;

procedure TPicturePopup.HttpCli1RequestDone(Sender: TObject;
  RqType: THttpRequest; Error: Word);
begin
  FHttpStream.Position := 0;
  jpgData.LoadFromStream(FHttpStream);
  Image1.Picture.Assign(jpgData);
end;

procedure TPicturePopup.btnResizeClick(Sender: TObject);
begin
  Image1.Stretch  :=  (not btnResize.Down);
  Image1.AutoSize :=  (btnResize.Down);
end;

procedure TPicturePopup.TrackBar1Change(Sender: TObject);
begin
  if  (TrackBar1.Position = TrackBar1.Max)  then  begin
    AlphaBlend  :=  False;
  end else begin
    AlphaBlend      :=  True;
    AlphaBlendValue :=  TrackBar1.Position + 55;
  end;
end;

procedure TPicturePopup.FormShow(Sender: TObject);
begin
  if  (WindowState = wsNormal)  then  begin
    Self.SetBounds( ZonuConfig.JpegPopupPosLeft,
                    ZonuConfig.JpegPopupPosTop,
                    ZonuConfig.JpegPopupWidth,
                    ZonuConfig.JpegPopupHeight);
    Self.TrackBar1.Position :=  ZonuConfig.JpegAlphaBlendValue;
    TrackBar1Change(Self);
  end;
end;

procedure TPicturePopup.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  SaveSetting;
  if  (ZonuConfig.JpegPopupOnTopStay) then  begin
    SetWindowPos(Handle, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOSIZE or SWP_NOMOVE);
  end;
end;

procedure TPicturePopup.SaveAs1Click(Sender: TObject);
begin
  if  (SaveDialog1.Execute) then  begin
    jpgData.SaveToFile(SaveDialog1.FileName);
  end;
end;

procedure TPicturePopup.FormDeactivate(Sender: TObject);
begin
  SaveSetting;
end;

procedure TPicturePopup.SaveSetting;
begin
  if  (WindowState = wsNormal)  then  begin
    ZonuConfig.JpegPopupPosLeft :=  Left;
    ZonuConfig.JpegPopupPosTop  :=  Top;
    ZonuConfig.JpegPopupWidth   :=  Width;
    ZonuConfig.JpegPopupHeight  :=  Height;
  end;
  ZonuConfig.JpegAlphaBlendValue  :=  Self.TrackBar1.Position;
end;

procedure TPicturePopup.FormActivate(Sender: TObject);
begin
  if  (ZonuConfig.JpegPopupOnTopStay) then  begin
    SetWindowPos(Handle, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE or SWP_NOMOVE);
  end else begin
    SetWindowPos(Handle, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOSIZE or SWP_NOMOVE);
  end;
  N2.Checked  :=  ZonuConfig.JpegPopupOnTopStay;
end;

procedure TPicturePopup.N2Click(Sender: TObject);
begin
  ZonuConfig.JpegPopupOnTopStay := (not ZonuConfig.JpegPopupOnTopStay);
  N2.Checked  :=  ZonuConfig.JpegPopupOnTopStay;
  FormActivate(Self);
end;

end.
