{ *************************************************************************** }
{                                                                             }
{ Hotzonu 15ch Script Unit                                                    }
{                                                                             }
{ 2001-04 Hotzonu Project, Monazilla Software Foundation 2001                 }
{                                                                             }
{ *************************************************************************** }

unit Zonu15ch;

interface

uses
  Classes, SysUtils, ZonuClass, ZonuCommon, ZonuHttp, Hzbconst;

type
  PParamData  = ^TParamData;
  TParamData  = record
    PageIndex : integer;
    ThreadItemCount : integer; //_E[hXbh̐
    Subject : string;
  end;

  T15chScript = class(TBBS)
  private
    { private }
    procedure _DownloadThreadList(Downloader: TThreadListDownloader;
      const NoCache, Async, InfSave: boolean);
  protected
    { protected }
    function GetThreadListType: TThreadListAccessType; override;
    function GetThreadDownloadMode: TThreadDownloadMode; override;
    function GetNeedHRefLink: boolean; override;
    function GetNeedLinkTarget: boolean; override;
    function GetRelativeLink: boolean; override;
    function GetIsPublished: boolean; override;
    procedure ZonuDataThreadList(Sender: TObject; Downloader: TDownloader;
      Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean); override;
    procedure ZonuEndThreadList(Sender: TObject; Downloader: TDownloader;
      Error: boolean); override;
    procedure ZonuThreadListError(Sender: TObject; Downloader: TDownloader;
      ErrorCode: integer; ErrorMsg: string); override;
    procedure ZonuDataRes(Sender: TObject; Downloader: TDownloader;
      Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean); override;
    procedure ZonuEndRes(Sender: TObject; Downloader: TDownloader;
      Error: boolean); override;
    procedure ZonuResError(Sender: TObject; Downloader: TDownloader;
      ErrorCode: integer; ErrorMsg: string); override;
  public
    { public }
    procedure ParseSubject(subject: string; out FileName: string;
      out Title: string; out Count: integer); override;
    procedure ParseDat(const ALine: string; const index: integer;
      var ARes: TRes; out ASubject: string); override;
    procedure DownloadThreadList(Downloader: TThreadListDownloader;
      const NoCache, Async, InfSave: boolean); override;
    procedure DownloadNextThreadList(Downloader: TThreadListDownloader;
      const PageIndex: integer); override;
    procedure DownloadThreadItem(Downloader: TResDownloader;
      const NoCache, Async, InfSave, AutoPilot: boolean); override;
    procedure SendRes(Uploader: TResUploader); override;
    procedure SendThread(Uploader: TThreadUploader); override;
    function  GetDisplayUrl(ThreadItem: TThreadItem): string; override;
    function  GetReadCgiUrl(ThreadItem: TThreadItem):string; override;
    function ConvUrl(const Url: string): string; override;
  end;

implementation

{ T15chScript }

//
//  Xbhꗗ̎擾@
//
function T15chScript.GetThreadListType: TThreadListAccessType;
begin
  Result  :=  tlOneByOne;
end;

//
//  Xbh̎擾@
//
function T15chScript.GetThreadDownloadMode: TThreadDownloadMode;
begin
  Result  :=  dmCGI;
end;

//
//  http://ւ̃Nǉ
//
function T15chScript.GetNeedHRefLink: boolean;
begin
  Result  :=  True;
end;

//
//  NւTarget=_blankǉ
//
function T15chScript.GetNeedLinkTarget: boolean;
begin
  Result  :=  False;
end;

//
//  BBSEBU[hŕ\邩ǂ
//
function T15chScript.GetIsPublished: boolean;
begin
  Result  :=  True;
end;

//
//  Xbhꗗ_E[h
//
procedure T15chScript.DownloadThreadList(Downloader: TThreadListDownloader;
  const NoCache, Async, InfSave: boolean);
var
  ParamData : PParamData;
begin
  New(ParamData);
  ParamData.PageIndex   :=  0;
  Downloader.ParamData  :=  ParamData;
  _DownloadThreadList(Downloader, NoCache, Async, InfSave);
end;

procedure T15chScript.DownloadNextThreadList(
  Downloader: TThreadListDownloader; const PageIndex: integer);
var
  ParamData : PParamData;
begin
  inherited;
  New(ParamData);
  ParamData.PageIndex   :=  PageIndex;
  Downloader.ParamData  :=  ParamData;
  _DownloadThreadList(Downloader, False, True, True);
end;

procedure T15chScript._DownloadThreadList(Downloader: TThreadListDownloader;
  const NoCache, Async, InfSave: boolean);
var
  ParamData : PParamData;
  FZonuHttp : TZonuHttp;
begin
  ParamData :=  PParamData(Downloader.ParamData);
  ParamData.ThreadItemCount :=  0;
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  Downloader;
  FZonuHttp.OnZonuData      :=  ZonuDataThreadList;
  FZonuHttp.OnZonuEnd       :=  ZonuEndThreadList;
  FZonuHttp.OnZonuError     :=  ZonuThreadListError;
  FZonuHttp.Url             :=  GetSubjectUrl(Downloader.Board, ParamData.PageIndex);
  FZonuHttp.NoCache         :=  NoCache;
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.InfSave         :=  InfSave;
  FZonuHttp.GetModified(Downloader.Board.CachePath + 'subject.inf', Async);
end;

//
//  Xbhꗗ擾
//
procedure T15chScript.ZonuDataThreadList(Sender: TObject;
  Downloader: TDownloader; Data: string; TotalSize,
  DownloadSize: integer; var MoreData: boolean);
var
  dl        : TThreadListDownloader;
  AFileName : string;
  ATitle    : string;
  ACount    : integer;
  ThreadItem: TThreadItem;
  ParamData : PParamData;
begin
  inherited;
  ParamData :=  PParamData(Downloader.ParamData);
  if  (AnsiPos('<b>', Data) = 1)  and
      (AnsiPos('<a href="./readres.cgi?', Data) > 0)  then
  begin
    if  (ParamData.PageIndex = 0) or
        ((ParamData.PageIndex > 0) and (ParamData.ThreadItemCount > 0)) then
    begin
      dl  :=  TThreadListDownloader(Downloader);
      ParseSubject(Data, AFileName, ATitle, ACount);
      ThreadItem                    :=  TThreadItem.Create;
      ThreadItem.BBS                :=  Self;
      ThreadItem.Site               :=  dl.Site;
      ThreadItem.BoardID            :=  dl.Board.BoardID;
      ThreadItem.No                 :=  0;
      ThreadItem.DatFileName        :=  AFileName;
      ThreadItem.Title              :=  ATitle;
      ThreadItem.AgeCount           :=  ACount;
      ThreadItem.RemoteDatFileName  :=  GetDatUrl(dl.Board, AFileName);
      ThreadItem.LocalDatFileName   :=  dl.Board.CachePath + AFileName;
      dl.FireDataEvt(ThreadItem, DownloadSize, TotalSize, MoreData);
    end;
    inc(ParamData.ThreadItemCount);
  end;
end;

//
//  Xbhꗗ擾I
//
procedure T15chScript.ZonuEndThreadList(Sender: TObject;
  Downloader: TDownloader; Error: boolean);
var
  ParamData:  PParamData;
begin
  ParamData :=  PParamData(Downloader.ParamData);
  Dispose(ParamData);
  inherited;
end;

//
//  Xbhꗗ擾G[
//
procedure T15chScript.ZonuThreadListError(Sender: TObject;
  Downloader: TDownloader; ErrorCode: integer; ErrorMsg: string);
begin
  inherited;

end;

//
//  Xbhꗗ̉
//
procedure T15chScript.ParseSubject(subject: string; out FileName,
  Title: string; out Count: integer);
begin
  inherited;
  Parse15chSubject(subject, FileName, Title, Count);
  FileName  :=  FileName  + '.dat';
end;

//
//  X̉
//
procedure T15chScript.ParseDat(const ALine: string; const index: integer;
  var ARes: TRes; out ASubject: string);
begin
  inherited;
  Parse15chDat(ALine, Index, ARes, ASubject);
end;

//
//  \URL
//
function T15chScript.GetDisplayUrl(ThreadItem: TThreadItem): string;
begin
  Result  :=  GetReadCgiUrl(ThreadItem);// + 'L50';
end;

//
//  Xbh̃_E[h
//
procedure T15chScript.DownloadThreadItem(Downloader: TResDownloader;
  const NoCache, Async, InfSave, AutoPilot: boolean);
var
  ParamData : PParamData;
  FZonuHttp : TZonuHttp;
begin
  New(ParamData);
  ParamData.Subject         :=  '';
  Downloader.ParamData      :=  ParamData;
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  Downloader;
  FZonuHttp.OnZonuData      :=  ZonuDataRes;
  FZonuHttp.OnZonuEnd       :=  ZonuEndRes;
  FZonuHttp.OnZonuError     :=  ZonuResError;
  FZonuHttp.Url             :=  Conv15chUrl2( Downloader.BBS.GetReadCgiUrl(Downloader.ThreadItem) );
  FZonuHttp.NoCache         :=  NoCache;
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.NeedDataEvent   :=  True;
  FZonuHttp.CacheReady      :=  False;
  FZonuHttp.InfSave         :=  InfSave;
  FZonuHttp.CacheReady      :=  Downloader.CacheReady;
  Downloader.HttpObj        :=  FZonuHttp;
  FZonuHttp.GetCgiDiff(ChangeFileExt(Downloader.ThreadItem.LocalDatFileName, '.inf'), Async);
end;

//
//  X擾
//
procedure T15chScript.ZonuDataRes(Sender: TObject; Downloader: TDownloader;
  Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean);
var
  dl  : TResDownloader;
  ParamData:  PParamData;
  ASubject :  string;
  Res : TRes;
  s   : string;
begin
  dl  :=  TResDownloader(Downloader);
  ParamData :=  PParamData(Downloader.ParamData);
  if  (AnsiPos('<br>@<dt>', Data) = 1)  or
      (AnsiPos('<dl>', Data) = 1)  then
  begin
    Res :=  TRes.Create;
    ParseDat(Data, 0, Res, ASubject);
    if  (Res.No = 1)  then  begin
      if  (dl.ResList.Count > 0)  then  begin
        //䕂͔͈͊ÕXԍNGXgƂPԂ̃XԂĂ
        Res.Free;
        Exit;
      end;
    end;
    Res.BBS         :=  dl.BBS;
    Res.Site        :=  dl.Site;
    Res.ThreadTitle :=  ParamData.Subject;
    if  (dl.ThreadItem.Title = '')  then  begin
      if  (ParamData.Subject <> '')  then  begin
        dl.ThreadItem.Title :=  ParamData.Subject;
      end;
    end;
    dl.FireDataEvt(Res, DownloadSize, TotalSize);
  end;
  if  (AnsiPos('<DL><B><FONT color=red size=+1>', Data) = 1)  then  begin
    s :=  Copy(Data, 32, Length(Data));
    ParamData.Subject  :=  Copy(s, 1, AnsiPos('</FONT>', s) - 1);
  end;
end;

//
//  X擾I
//
procedure T15chScript.ZonuEndRes(Sender: TObject; Downloader: TDownloader;
  Error: boolean);
var
  ParamData:  PParamData;
begin
  ParamData :=  PParamData(Downloader.ParamData);
  Dispose(ParamData);
  inherited;
end;

//
//  X擾G[
//
procedure T15chScript.ZonuResError(Sender: TObject;
  Downloader: TDownloader; ErrorCode: integer; ErrorMsg: string);
begin
  inherited;

end;

//
//  
//
procedure T15chScript.SendRes(Uploader: TResUploader);
var
  FZonuHttp : TZonuHttp;
begin
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Uploader        :=  Uploader;
  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;
  FZonuHttp.Url             :=  GetSendUrl(Uploader.Board, Uploader.ThreadItem);
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.SendReferer     :=  Uploader.BBS.GetReadCgiUrl(Uploader.ThreadItem);
  FZonuHttp.SendContent     :=  Uploader.SendData.ConvSendContent(GetSendParam);
  FZonuHttp.SendCookie      :=  Uploader.SendData.ConvSendContent(GetSendCookie);
  FZonuHttp.Send;
end;

//
//  VX
//
procedure T15chScript.SendThread(Uploader: TThreadUploader);
var
  FZonuHttp   : TZonuHttp;
begin
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Uploader        :=  Uploader;
  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;
  FZonuHttp.Url             :=  GetNewThreadUrl(Uploader.Board);
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.SendReferer     :=  Uploader.Board.Url;
  FZonuHttp.SendContent     :=  Uploader.SendData.ConvSendContent(GetNewThreadParam);
  FZonuHttp.SendCookie      :=  Uploader.SendData.ConvSendContent(GetNewThreadCookie);
  FZonuHttp.Send;
end;

function T15chScript.GetReadCgiUrl(ThreadItem: TThreadItem): string;
var
  s: string;
begin
  s :=  inherited GetReadCgiUrl(ThreadItem);
  Result  :=  Conv15chUrl(s);
end;

function T15chScript.GetRelativeLink: boolean;
begin
  Result  :=  True;
end;

//
//  URL PATHINFO^^ɕϊ
//
function T15chScript.ConvUrl(const Url: string): string;
begin
  Result  :=  Conv15chUrl(Url);
end;

end.
