{ *************************************************************************** }
{                                                                             }
{ Hotzonu megabbs Script Unit                                                 }
{                                                                             }
{ 2001-04 Hotzonu Project, Monazilla Software Foundation 2001                 }
{                                                                             }
{ *************************************************************************** }

unit ZonuMegabbs;

interface

uses
  Classes, SysUtils, ZonuClass, ZonuCommon, ZonuHttp, Hzbconst;

type
  PParamData  = ^TParamData;
  TParamData  = record
    Subject : string;
    Crypt:  string;
  end;

  TMegabbsScript = class(TBBS)
  private
    { private }
  protected
    { protected }
    function GetThreadListType: TThreadListAccessType; override;
    function GetThreadDownloadMode: TThreadDownloadMode; override;
    function GetNeedHRefLink: boolean; override;
    function GetNeedLinkTarget: boolean; override;
    function GetRelativeLink: boolean; override;
    function GetIsPublished: boolean; override;
    procedure ZonuDataThreadList(Sender: TObject; Downloader: TDownloader;
      Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean); override;
    procedure ZonuEndThreadList(Sender: TObject; Downloader: TDownloader;
      Error: boolean); override;
    procedure ZonuThreadListError(Sender: TObject; Downloader: TDownloader;
      ErrorCode: integer; ErrorMsg: string); override;
    procedure ZonuDataRes(Sender: TObject; Downloader: TDownloader;
      Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean); override;
    procedure ZonuEndRes(Sender: TObject; Downloader: TDownloader;
      Error: boolean); override;
    procedure ZonuResError(Sender: TObject; Downloader: TDownloader;
      ErrorCode: integer; ErrorMsg: string); override;
  public
    { public }
    procedure ParseSubject(subject: string; out FileName: string;
      out Title: string; out Count: integer); override;
    procedure ParseDat(const ALine: string; const index: integer;
      var ARes: TRes; out ASubject: string); override;
    procedure DownloadThreadList(Downloader: TThreadListDownloader;
      const NoCache, ASync, InfSave: boolean); override;
    procedure DownloadThreadItem(Downloader: TResDownloader;
      const NoCache, ASync, InfSave, AutoPilot: boolean); override;
    procedure SendRes(Uploader: TResUploader); override;
    procedure SendThread(Uploader: TThreadUploader); override;
    function  GetDisplayUrl(ThreadItem: TThreadItem): string; override;
    function  GetCryptValue(Board: TBoard): string;
  end;

implementation

{ TMegabbsScript }

//
//  Xbhꗗ̎擾@
//
function TMegabbsScript.GetThreadListType: TThreadListAccessType;
begin
  Result  :=  tlLump;
end;

//
//  Xbh̎擾@
//
function TMegabbsScript.GetThreadDownloadMode: TThreadDownloadMode;
begin
  Result  :=  dmCGI;
end;

//
//  http://ւ̃Nǉ
//
function TMegabbsScript.GetNeedHRefLink: boolean;
begin
  Result  :=  True;
end;

//
//  NւTarget=_blankǉ
//
function TMegabbsScript.GetNeedLinkTarget: boolean;
begin
  Result  :=  True;
end;

//
//  BBSEBU[hŕ\邩ǂ
//
function TMegabbsScript.GetIsPublished: boolean;
begin
  Result  :=  True;
end;

//
//  Xbhꗗ_E[h
//
procedure TMegabbsScript.DownloadThreadList(Downloader: TThreadListDownloader;
  const NoCache, ASync, InfSave: boolean);
var
  ParamData : PParamData;
  FZonuHttp : TZonuHttp;
begin
  New(ParamData);
  Downloader.ParamData      :=  ParamData;
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  Downloader;
  FZonuHttp.OnZonuData      :=  ZonuDataThreadList;
  FZonuHttp.OnZonuEnd       :=  ZonuEndThreadList;
  FZonuHttp.OnZonuError     :=  ZonuThreadListError;
  FZonuHttp.Url             :=  GetSubjectUrl(Downloader.Board);
  FZonuHttp.NoCache         :=  NoCache;
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.InfSave         :=  InfSave;
  FZonuHttp.GetModified(Downloader.Board.CachePath + 'subject.inf', ASync);
end;

//
//  Xbhꗗ擾
//
procedure TMegabbsScript.ZonuDataThreadList(Sender: TObject;
  Downloader: TDownloader; Data: string; TotalSize,
  DownloadSize: integer; var MoreData: boolean);
var
  dl        : TThreadListDownloader;
  AFileName : string;
  ATitle    : string;
  ACount    : integer;
  ThreadItem: TThreadItem;
begin
  inherited;
  if  (AnsiPos('<a href="readres.cgi?', Data) = 1)  then  begin
    dl  :=  TThreadListDownloader(Downloader);
    ParseSubject(Data, AFileName, ATitle, ACount);
    ThreadItem                    :=  TThreadItem.Create;
    ThreadItem.BBS                :=  Self;
    ThreadItem.Site               :=  dl.Site;
    ThreadItem.BoardID            :=  dl.Board.BoardID;
    ThreadItem.No                 :=  0;
    ThreadItem.DatFileName        :=  AFileName;
    ThreadItem.Title              :=  ATitle;
    ThreadItem.AgeCount           :=  ACount;
    ThreadItem.RemoteDatFileName  :=  GetDatUrl(dl.Board, AFileName);
    ThreadItem.LocalDatFileName   :=  dl.Board.CachePath + AFileName + '.dat';
    dl.FireDataEvt(ThreadItem, DownloadSize, TotalSize, MoreData);
  end;
end;

//
//  Xbhꗗ擾I
//
procedure TMegabbsScript.ZonuEndThreadList(Sender: TObject;
  Downloader: TDownloader; Error: boolean);
var
  ParamData:  PParamData;
begin
  ParamData :=  PParamData(Downloader.ParamData);
  Dispose(ParamData);
  inherited;
end;

//
//  Xbhꗗ擾G[
//
procedure TMegabbsScript.ZonuThreadListError(Sender: TObject;
  Downloader: TDownloader; ErrorCode: integer; ErrorMsg: string);
begin
  inherited;

end;

//
//  Xbhꗗ̉
//
procedure TMegabbsScript.ParseSubject(subject: string; out FileName,
  Title: string; out Count: integer);
begin
  inherited;
  ParseMegabbsSubject(subject, FileName, Title, Count);
end;

//
//  X̉
//
procedure TMegabbsScript.ParseDat(const ALine: string;
  const index: integer; var ARes: TRes; out ASubject: string);
begin
  inherited;
  ParseMegabbsDat(ALine, Index, ARes, ASubject);
end;

//
//  \URL
//
function TMegabbsScript.GetDisplayUrl(ThreadItem: TThreadItem): string;
begin
  Result  :=  GetReadCgiUrl(ThreadItem) + '&rm=100';
end;

//
//  Xbh̃_E[h
//
procedure TMegabbsScript.DownloadThreadItem(Downloader: TResDownloader;
  const NoCache, ASync, InfSave, AutoPilot: boolean);
var
  ParamData : PParamData;
  FZonuHttp : TZonuHttp;
begin
  New(ParamData);
  ParamData.Subject         :=  '';
  ParamData.Crypt           :=  '';
  Downloader.ParamData      :=  ParamData;
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  Downloader;
  FZonuHttp.OnZonuData      :=  ZonuDataRes;
  FZonuHttp.OnZonuEnd       :=  ZonuEndRes;
  FZonuHttp.OnZonuError     :=  ZonuResError;
  FZonuHttp.Url             :=  Downloader.BBS.GetReadCgiUrl(Downloader.ThreadItem);
  FZonuHttp.NoCache         :=  NoCache;
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.NeedDataEvent   :=  True;
  FZonuHttp.CacheReady      :=  False;
  FZonuHttp.InfSave         :=  InfSave;
  FZonuHttp.CacheReady      :=  Downloader.CacheReady;
  Downloader.HttpObj        :=  FZonuHttp;
  FZonuHttp.GetCgiDiff(ChangeFileExt(Downloader.ThreadItem.LocalDatFileName, '.inf'), ASync);
end;

//
//  X擾
//
procedure TMegabbsScript.ZonuDataRes(Sender: TObject;
  Downloader: TDownloader; Data: string; TotalSize, DownloadSize: integer;
  var MoreData: boolean);
var
  dl  : TResDownloader;
  ParamData:  PParamData;
  ASubject :  string;
  Res : TRes;
  s   : string;
begin
  dl  :=  TResDownloader(Downloader);
  ParamData :=  PParamData(Downloader.ParamData);
  if  (AnsiPos('<DT>', Data) = 1) and (AnsiPos('<DD>', Data) = 0) then  begin
    dl.Temp :=  Data;
  end else
  if  ((AnsiPos('<DD>', Data) = 1) and (dl.Temp <> ''))  or
      ((AnsiPos('<DT>', Data) = 1) and (AnsiPos('<DD>', Data) > 0))  then
  begin
    Res :=  TRes.Create;
    ParseDat(dl.Temp + Data, 0, Res, ASubject);
    Res.BBS         :=  dl.BBS;
    Res.Site        :=  dl.Site;
    Res.ThreadTitle :=  ParamData.Subject;
    if  (dl.ThreadItem.Title = '')  then  begin
      if  (ParamData.Subject <> '')  then  begin
        dl.ThreadItem.Title :=  ParamData.Subject;
      end;
    end;
    dl.FireDataEvt(Res, DownloadSize, TotalSize);
    dl.Temp :=  '';
  end else
  if  (AnsiPos('<input type=hidden name=crypt', Data) = 1) then  begin
    s :=  Copy(Data, AnsiPos('value=', Data) + 6, Length(Data));
    s :=  Copy(S, 1, Length(S) - 1);
    ParamData.Crypt :=  S;
  end else
  if  (AnsiPos('<title>', Data) = 1) then  begin
    s :=  Copy(Data, 8, Length(Data));
    ParamData.Subject  :=  Copy(s, 1, AnsiPos('</title>', s) - 1);
  end else begin
    dl.Temp :=  '';
  end;
end;

//
//  X擾I
//
procedure TMegabbsScript.ZonuEndRes(Sender: TObject;
  Downloader: TDownloader; Error: boolean);
var
  ParamData:  PParamData;
begin
  ParamData :=  PParamData(Downloader.ParamData);
  TResDownloader(Downloader).ThreadItem.Crypt :=  ParamData.Crypt;
  Dispose(ParamData);
  inherited;
end;

//
//  X擾G[
//
procedure TMegabbsScript.ZonuResError(Sender: TObject;
  Downloader: TDownloader; ErrorCode: integer; ErrorMsg: string);
begin
  inherited;

end;

//
//  
//
procedure TMegabbsScript.SendRes(Uploader: TResUploader);
var
  FZonuHttp : TZonuHttp;
begin
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Uploader        :=  Uploader;
  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;
  FZonuHttp.Url             :=  GetSendUrl(Uploader.Board, Uploader.ThreadItem);
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.SendReferer     :=  Uploader.BBS.GetReadCgiUrl(Uploader.ThreadItem);
  FZonuHttp.SendContent     :=  Uploader.SendData.ConvSendContent(GetSendParam);
  FZonuHttp.SendCookie      :=  Uploader.SendData.ConvSendContent(GetSendCookie);
  FZonuHttp.Send;
end;

//
//  VX
//
procedure TMegabbsScript.SendThread(Uploader: TThreadUploader);
var
  FZonuHttp   : TZonuHttp;
begin
  //̃y[WCryptl擾(megabbs only)
  Uploader.SendData.Crypt   :=  GetCryptValue(Uploader.Board);

  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Uploader        :=  Uploader;
  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;
  FZonuHttp.Url             :=  GetNewThreadUrl(Uploader.Board);
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.SendReferer     :=  Uploader.Board.Url;
  FZonuHttp.SendContent     :=  Uploader.SendData.ConvSendContent(GetNewThreadParam);
  FZonuHttp.SendCookie      :=  Uploader.SendData.ConvSendContent(GetNewThreadCookie);
  FZonuHttp.Send;
end;

function TMegabbsScript.GetCryptValue(Board: TBoard): string;
var
  FZonuHttp   : TZonuHttp;
  FDownloader : TDownloader;
  sl: TStringList;
  s: string;
  i: integer;
begin
  s   :=  '';
  sl  :=  TStringList.Create;
  FDownloader               :=  TDownloader.Create;
  FDownloader.BBS           :=  Board.BBS;
  FDownloader.Site          :=  Board.Site;
  FDownloader.CharSet       :=  Board.CharSet;

  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  FDownloader;
  FZonuHttp.Url             :=  Board.Url;
  FZonuHttp.NoCache         :=  True;
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.InfSave         :=  False;
  sl.Text :=  FZonuHttp.GetNotAsync;
  for i :=  0 to  sl.Count - 1  do  begin
    if  (AnsiPos('name=crypt', AnsiLowerCase(sl[i])) > 0)  and
        (AnsiPos('<input', AnsiLowerCase(sl[i])) > 0)  then
    begin
      s :=  Copy(sl[i], AnsiPos('value=', AnsiLowerCase(sl[i])) + 6, Length(sl[i]));
      s :=  Copy(s, 1, AnsiPos('>', s) - 1);
      Break; 
    end;
  end;
  sl.Free;
  FZonuHttp.Free;
  FDownloader.Free;
  Result  :=  s;
end;

function TMegabbsScript.GetRelativeLink: boolean;
begin
  Result  :=  False;
end;

end.
